"""Evergreen API Module."""
# Shortcuts for importing.
from evergreen.api import CachedEvergreenApi, EvergreenApi, Requester, RetryingEvergreenApi
from evergreen.api_requests import IssueLinkRequest
from evergreen.build import Build
from evergreen.commitqueue import CommitQueue
from evergreen.distro import Distro
from evergreen.host import Host
from evergreen.manifest import Manifest
from evergreen.patch import Patch
from evergreen.project import Project
from evergreen.stats import TaskStats, TestStats
from evergreen.task import Task
from evergreen.task_annotations import TaskAnnotation
from evergreen.task_reliability import TaskReliability
from evergreen.tst import Tst
from evergreen.version import Version



"""Representation of evergreen manifest."""
from __future__ import absolute_import

from typing import TYPE_CHECKING, Any, Dict, Optional

from evergreen.base import _BaseEvergreenObject, evg_attrib

if TYPE_CHECKING:
    from evergreen.api import EvergreenApi


class ManifestModule(_BaseEvergreenObject):
    """Represents a module in the evergreen manifest."""

    branch = evg_attrib("branch")
    repo = evg_attrib("repo")
    revision = evg_attrib("revision")
    owner = evg_attrib("owner")
    url = evg_attrib("url")

    def __init__(self, name: str, json: Dict[str, Any], api: "EvergreenApi") -> None:
        """
        Create an instance of an evergreen manifest module.

        :param json: json representing manifest.
        :param api: evergreen api object
        """
        super(ManifestModule, self).__init__(json, api)
        self.name = name


class Manifest(_BaseEvergreenObject):
    """Representation of an evergreen manifest."""

    id = evg_attrib("id")
    revision = evg_attrib("revision")
    project = evg_attrib("project")
    branch = evg_attrib("branch")

    def __init__(self, json: Dict[str, Any], api: "EvergreenApi") -> None:
        """
        Create an instance of an evergreen version manifest.

        :param json: json representing manifest.
        :param api: evergreen api object.
        """
        super(Manifest, self).__init__(json, api)

    @property
    def modules(self) -> Optional[Dict[str, ManifestModule]]:
        """Map of modules in this manifest."""
        if "modules" not in self.json:
            return {}

        modules = self.json["modules"].items()

        return {
            module_key: ManifestModule(module_key, module_value, self._api)
            for module_key, module_value in modules
        }



"""Objects for making requests to the API."""
from typing import Dict, List, NamedTuple, Optional

from pydantic import BaseModel


class IssueLinkRequest(NamedTuple):
    """Issue to add to a task annotation."""

    issue_key: str
    url: str

    def as_dict(self) -> Dict[str, str]:
        """Get a dictionary representation of the issue link."""
        return {"issue_key": self.issue_key, "url": self.url}


class SlackAttachmentField(BaseModel):
    """
    Slack fields that get displayed in a table-like format.

    title: The field title.
    value: The field text. It can be formatted as plain text or with markdown by using mrkdwn_in.
    short: Indicates whether the field object is short enough to be displayed side-by-side with
    other field objects.
    """

    title: Optional[str]
    value: Optional[str]
    short: Optional[bool]


class SlackAttachment(BaseModel):
    """
    An attachment to be sent using Slack.

    title: The attachment title.
    title_link: A URL that turns the title into a link.
    text: If fallback is empty, this is required. The main body text of the attachment as plain
    text, or with markdown using mrkdwn_in.
    fallback: If text is empty, this is required. A plain text summary of an attachment for
    clients that don't show formatted text (eg. IRC, mobile notifications).
    mrkdwn_in: An array of fields that should be formatted with markdown.
    color: The message color. Can either be one of good (green), warning (yellow), danger (red),
    or any hex color code (eg. #439FE0).
    author_name: The display name of the author.
    author_icon: A URL that displays the author icon. Will only work if author_name is present.
    fields: Array of SlackAttachmentFields that get displayed in a table-like format.
    """

    title: Optional[str]
    title_link: Optional[str]
    text: Optional[str]
    fallback: Optional[str]
    mrkdwn_in: Optional[List[str]]
    color: Optional[str]
    author_name: Optional[str]
    author_icon: Optional[str]
    fields: Optional[List[SlackAttachmentField]]



# -*- encoding: utf-8 -*-
"""Representation of an evergreen build."""
from __future__ import absolute_import

from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_datetime_attrib
from evergreen.metrics.buildmetrics import BuildMetrics

if TYPE_CHECKING:
    from evergreen.api import EvergreenApi
    from evergreen.task import Task  # noqa: F401
    from evergreen.version import Version

EVG_BUILD_STATUS_FAILED = "failed"
EVG_BUILD_STATUS_SUCCESS = "success"
EVG_BUILD_STATUS_CREATED = "created"

COMPLETED_STATES = {
    EVG_BUILD_STATUS_FAILED,
    EVG_BUILD_STATUS_SUCCESS,
}


class StatusCounts(_BaseEvergreenObject):
    """Representation of Evergreen StatusCounts."""

    succeeded = evg_attrib("succeeded")
    failed = evg_attrib("failed")
    started = evg_attrib("started")
    undispatched = evg_attrib("undispatched")
    inactivate = evg_attrib("inactive")
    dispatched = evg_attrib("dispatched")
    timed_out = evg_attrib("timed_out")

    def __init__(self, json: Dict[str, Any], api: "EvergreenApi") -> None:
        """
        Create a Status Counts object.

        :param json: Json of status counts object.
        :param api: Evergreen API.
        """
        super(StatusCounts, self).__init__(json, api)


class Build(_BaseEvergreenObject):
    """Representation of an Evergreen build."""

    id = evg_attrib("_id")
    project_id = evg_attrib("project_id")
    create_time = evg_datetime_attrib("create_time")
    start_time = evg_datetime_attrib("start_time")
    finish_time = evg_datetime_attrib("finish_time")
    version = evg_attrib("version")
    branch = evg_attrib("branch")
    git_hash = evg_attrib("git_hash")
    build_variant = evg_attrib("build_variant")
    status = evg_attrib("status")
    activated = evg_attrib("activated")
    activated_by = evg_attrib("activated_by")
    activated_time = evg_datetime_attrib("activated_time")
    order = evg_attrib("order")
    tasks = evg_attrib("tasks")
    time_taken_ms = evg_attrib("time_taken_ms")
    display_name = evg_attrib("display_name")
    predicted_makespan_ms = evg_attrib("predicted_makespan_ms")
    actual_makespan_ms = evg_attrib("actual_makespan_ms")
    origin = evg_attrib("origin")

    def __init__(self, json: Dict[str, Any], api: "EvergreenApi") -> None:
        """
        Create an instance of an evergreen task.

        :param json: Json of build object.
        :param api: Evergreen API.
        """
        super(Build, self).__init__(json, api)

    @property
    def status_counts(self) -> StatusCounts:
        """Get the status counts of the build."""
        return StatusCounts(self.json["status_counts"], self._api)

    def get_project_identifier(self) -> str:
        """
        Retrieve the human-readable project id from Evergreen.

        This requires the caller to be Admin on the project.

        :return: Human-readable project id.
        """
        return self._api.project_by_id(self.project_id).identifier

    def get_tasks(self, fetch_all_executions: bool = False) -> List["Task"]:
        """
        Get all tasks for this build.

        :param fetch_all_executions:  fetch all executions for tasks.
        :return: List of all tasks.
        """
        return self._api.tasks_by_build(self.id, fetch_all_executions)

    def is_completed(self) -> bool:
        """
        Determine if this build has completed running tasks.

        :return: True if build has completed running tasks.
        """
        return self.status in COMPLETED_STATES

    def get_metrics(self, task_filter_fn: Callable = None) -> Optional[BuildMetrics]:
        """
        Get metrics for the build.

        Metrics are only available on build that have finished running..

        :param task_filter_fn: function to filter tasks included for metrics, should accept a task
                               argument.
        :return: Metrics for the build.
        """
        if self.status != EVG_BUILD_STATUS_CREATED:
            return BuildMetrics(self).calculate(task_filter_fn)
        return None

    def get_version(self) -> "Version":
        """
        Get the version this build is a part of.

        :return: Version that this build is a part of.
        """
        return self._api.version_by_id(self.version)

    def __repr__(self) -> str:
        """
        Get a string representation of Task for debugging purposes.

        :return: String representation of Task.
        """
        return "Build({id})".format(id=self.id)



# -*- encoding: utf-8 -*-
"""Evergreen representation of a project."""
from __future__ import absolute_import

from typing import TYPE_CHECKING, Any, Dict

from evergreen.base import _BaseEvergreenObject, evg_attrib
from evergreen.version import Version

if TYPE_CHECKING:
    from evergreen.api import EvergreenApi


class Project(_BaseEvergreenObject):
    """Representation of an Evergreen project."""

    batch_time = evg_attrib("batch_time")
    branch_name = evg_attrib("branch_name")
    display_name = evg_attrib("display_name")
    enabled = evg_attrib("enabled")
    identifier = evg_attrib("identifier")
    owner_name = evg_attrib("owner_name")
    private = evg_attrib("private")
    remote_path = evg_attrib("remote_path")
    repo_name = evg_attrib("repo_name")
    tracked = evg_attrib("tracked")
    deactivated_previous = evg_attrib("deactivate_previous")
    admins = evg_attrib("admins")
    tracks_push_events = evg_attrib("tracks_push_events")
    pr_testing_enabled = evg_attrib("pr_testing_enabled")
    commit_queue = evg_attrib("commit_queue")

    def __init__(self, json: Dict[str, Any], api: "EvergreenApi") -> None:
        """
        Create an instance of an evergreen project.

        :param json: json representing project.
        :param api: evergreen api object.
        """
        super(Project, self).__init__(json, api)

    def __str__(self) -> str:
        """Get a string version of the Project."""
        return self.identifier

    def most_recent_version(self) -> Version:
        """
        Fetch the most recent version.

        :return: Version queried for.
        """
        version_iterator = self._api.versions_by_project(self.identifier)
        return next(version_iterator)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['evergreen', 'evergreen.cli', 'evergreen.errors', 'evergreen.metrics']

package_data = \
{'': ['*']}

install_requires = \
['Click>=7,<8',
 'PyYAML>=5,<6',
 'pydantic>=1',
 'python-dateutil>=2,<3',
 'requests>=2,<3',
 'structlog>=19',
 'tenacity>=5']

entry_points = \
{'console_scripts': ['evg-api = evergreen.cli.main:main']}

setup_kwargs = {
    'name': 'evergreen.py',
    'version': '3.0.9',
    'description': 'Python client for the Evergreen API',
    'long_description': '# Evergreen.py\n\nA client library for the Evergreen API written in python. Currently supports the V2 version of\nthe API. For more details, see https://github.com/evergreen-ci/evergreen/wiki/REST-V2-Usage .\n\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/evergreen.py) [![PyPI](https://img.shields.io/pypi/v/evergreen.py.svg)](https://pypi.org/project/evergreen.py/) [![Coverage Status](https://coveralls.io/repos/github/evergreen-ci/evergreen.py/badge.svg?branch=master)](https://coveralls.io/github/evergreen-ci/evergreen.py?branch=master)\n\n## Documentation\n\nYou can find the documentation [here](https://evergreen-ci.github.io/evergreen.py/).\n\n## Usage\n\n```\n>>> from evergreen.api import EvgAuth, EvergreenApi\n>>> api = EvergreenApi.get_api(EvgAuth(\'david.bradford\', \'***\'))\n>>> project = api.project_by_id(\'mongodb-mongo-master\')\n>>> project.display_name\n\'MongoDB (master)\'\n```\n\n### Command Line Application\n\nA command line application is included to explore the evergreen api data. It is called `evg-api`.\n\n```\n$ evg-api --json list-hosts\n{\n    "host_id": "host num 0",\n    "host_url": "host.num.com",\n    "distro": {\n        "distro_id": "ubuntu1804-build",\n        "provider": "static",\n        "image_id": ""\n    },\n    "provisioned": true,\n    "started_by": "mci",\n    "host_type": "",\n    "user": "mci-exec",\n    "status": "running",\n    "running_task": {\n        "task_id": null,\n        "name": null,\n        "dispatch_time": null,\n        "version_id": null,\n        "build_id": null\n    },\n    "user_host": false\n}\n...\n```\n\nIt may also be used from inside the repo using\n```\n$ poetry install\n$ poetry run python src/evergreen/cli/main.py\n```\n\n## Contributors Guide\n\n### Testing\n\nUse poetry and pytest for testing.\n```\n$ poetry install\n$ poetry run pytest\n```\n\nTo get code coverage information:\n\n```\n$ poetry run pytest --cov=src --cov-report=html\n```\n\nThis will generate an html coverage report in `htmlcov/` directory.\n\nThere are a few tests that are slow running. These tests are not run by default, but can be included\nby setting the env variable RUN_SLOW_TESTS to any value.\n\n```\n$ RUN_SLOW_TEST=1 poetry run pytest\n```\n### Running checks on commit\n\nThis project has [pre-commit](https://pre-commit.com/) configured. Pre-commit will run configured\nchecks at git commit time. To enable pre-commit on your local repository run:\n\n```bash\n$ poetry run pre-commit install\n```\n\n### Versioning and Deploy\n\nBefore deploying a new version, please update the `CHANGELOG.md` file with a description of what\nis being changed.\n\nDeploys to [PyPi](https://pypi.org/project/evergreen.py/) are done automatically on merges to master.\nIn order to avoid overwriting a previous deploy, the version should be updated on all changes. The\n[semver](https://semver.org/) versioning scheme should be used for determining the version number.\n\nThe version is found in the `pyproject.toml` file.\n\n### Merging\n\nMerges to master should be done by the evergreen [commit queue](https://github.com/evergreen-ci/evergreen/wiki/Commit-Queue#pr).\nAfter a PR has been reviewed, add a comment with the text `evergreen merge` to merge the PR.\n',
    'author': 'Alexander Costas',
    'author_email': 'alexander.costas@mongodb.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/evergreen-ci/evergreen.py',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
